package stx.libjava.tests;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import junit.framework.TestCase;

import org.junit.runner.JUnitCore;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;

import stx.libjava.tests.mocks.EnumValueAnnotation;
import stx.libjava.tests.mocks.HeavilyAnnotatedClass;
import stx.libjava.tests.mocks.Weather;

public class AnnotationTests extends TestCase {

	public void testClassAnnotations() {
		Annotation[] annotations = HeavilyAnnotatedClass.class.getAnnotations();
		for (Annotation a : annotations) {
			System.out.println(a.annotationType());
		}
		assertTrue(annotations.length == 4);
	}
	
	public void testFieldAnnotations() throws Exception  {
		Field field = new HeavilyAnnotatedClass().getClass().getDeclaredField("heavilyAnnotatedField");
		Annotation[] annotations = field.getAnnotations();
		for (Annotation a : annotations) {
			System.out.println(a.annotationType());
		}
		assertTrue(annotations.length == 4);
	}
	
	public void testMethodAnnotations() throws Exception  {
		Method method = new HeavilyAnnotatedClass().getClass().getDeclaredMethod("heavilyAnnotatedMethod", String.class);
		Annotation[] annotations = method.getAnnotations();
		for (Annotation a : annotations) {
			System.out.println(a.annotationType());
		}
		assertTrue(annotations.length == 4);
	}
	
	public void testMethodParamAnnotations() throws Exception  {
		Method method = new HeavilyAnnotatedClass().getClass().getDeclaredMethod("heavilyAnnotatedMethod", String.class);
		Annotation[][] parameterAnnotations = method.getParameterAnnotations();
		Class[] parameterTypes = method.getParameterTypes();

		int i=0;
		for(Annotation[] annotations : parameterAnnotations){
		  Class parameterType = parameterTypes[i++];

		  for(Annotation annotation : annotations){
			  
		  assertTrue(annotation instanceof EnumValueAnnotation);

		        EnumValueAnnotation myAnnotation = (EnumValueAnnotation) annotation;
		        		       
		        assertTrue(myAnnotation.value().equals(Weather.SUNNY));
		    }
		}

	}
	
	public static void main(String[] args) {
		System.out.println("Expected output: 4 run, 0 failed");
		Result result = runAllTests();
		System.out.println("getRunCount() = " + result.getRunCount());
		System.out.println("getFailureCount() = " + result.getFailureCount());
		for (Failure f : result.getFailures()) {
			System.out.println("Failure: " + f.toString());	
		}
	}
	
	public static Result runAllTests() {		
		return JUnitCore.runClasses(AnnotationTests.class);			
	}
	
}
